import matplotlib.pyplot as plt
import moss
import numpy as np
import os
import pandas as pd
import seaborn as sns
sns.set_style('white')


class ModelWrapper(object):
    def __init__(self, designs):
        '''Model Design Abstraction

        Inputs: designs: Either a pandas dataframe or a filename (or list of filenames)
        of onset design files (containign at least 'onset' and 'condition' columns)

        Properties:

        designFiles : List of Input Design File (or None if initialized w/ a
                         dataframe)
        * mat : Moss GLM Design
        * designDf : Pandas df rows are timepoints
        '''

        if isinstance(designs, pd.DataFrame):
            self.designFiles = None
            self.designDf = designs
            self._basename = None
        else:
            self.designFiles = designs
            designList = []
            runDur = None
            for i, designFile in enumerate(designs):
                runDf = pd.read_csv(designFile)
                if not runDur:
                    runDur = runDf['onset'].max()
                if 'run' in runDf.columns and len(runDf['run'].unique()) == 1:
                    # Assume 1-base run index and add total run duration
                    runDf['onset'] += runDur * (runDf['run'][0] - 1)
                designList.append(runDf)
            self.designDf = pd.concat(designList)
            self._basename = os.path.splitext(os.path.commonprefix(designs))[0]

        self.mat, self.design = self.buildDesign(self.designDf)
        self.corrFrame = self.buildCorrFrame()

    def buildDesign(self, df, tr=2):
        maxTime = int(df['onset'].max())
        ntp = maxTime / tr

        mat = moss.glm.DesignMatrix(design=df, ntp=ntp, tr=tr)
        design = mat.design_matrix

        # Remove zero-columns (runEnd)
        design = design[design.columns[(design != 0).any()]]

        return mat, design

    def _saveDesign(self, outfile=None):
        if not outfile:
            outfile = self.basename + '_designmatrix.csv'
        print('saving moss glm as csv matrix %s' % outfile)
        self.design.to_csv(outfile)

    def buildCorrFrame(self, cols=None):
        if not cols:
            cols = self.design.columns.values
        return pd.DataFrame(
            np.corrcoef(self.mat.design_matrix[cols].T),
            columns=cols,
            index=cols)

    def _saveFigure(self, outfile=None):
        fig, ax = plt.subplots(1, 2, figsize=(10, 5))

        # Plot Design Timeseries
        sns.heatmap(self.design,
                    cmap='gray',
                    yticklabels=False,
                    ax=ax[0],
                    cbar=False)
        ax[0].set_xticklabels(ax[0].xaxis.get_majorticklabels(), rotation=90)

        # Plot Correlation
        sns.heatmap(self.corrFrame, annot=True, ax=ax[1], cbar=True)
        plt.yticks(rotation=0)
        plt.xticks(rotation=90)
        plt.tight_layout()

        if not outfile:
            outfile = self.basename + '_Collinearity.png'

        fig.savefig(outfile)

    @property
    def basename(self):
        if self._basename:
            base = self._basename
        else:
            raise StandardError("No default basename for pandas dataframes")
        return base

    @basename.setter
    def basename(self, value):
        self._basename = value



import glob
import orrey
import os
import pandas as pd
from unittest import TestCase

filepath = os.path.abspath(__file__)
basedir = os.path.dirname(filepath)

class TestModelCorr(TestCase):
    def test_builds_df_with_one_design_file(self):
        fixtures = glob.glob(os.path.join(basedir, 'fixtures/*design.csv'))[:1]
        mod = orrey.model.ModelWrapper(fixtures)
        self.assertTrue(isinstance(mod.designDf, pd.DataFrame))
        self.assertEqual(mod.design.shape[0], 120)

    def test_builds_df_with_two_design_files(self):
        fixtures = glob.glob(os.path.join(basedir, 'fixtures/*design.csv'))
        mod = orrey.model.ModelWrapper(fixtures)
        self.assertEqual(mod.design.shape[0], 240)

    def test_saves_design(self):
        fixtures = glob.glob(os.path.join(basedir, 'fixtures/*design.csv'))[:1]
        mod = orrey.model.ModelWrapper(fixtures)
        outfile = 'tmp.csv'
        mod._saveDesign(outfile=outfile)
        self.assertTrue(os.path.exists(outfile))

    def test_saves_png(self):
        fixtures = glob.glob(os.path.join(basedir, 'fixtures/*design.csv'))[:1]
        mod = orrey.model.ModelWrapper(fixtures)
        outfile = 'tmp.png'
        mod._saveFigure(outfile=outfile)
        self.assertTrue(os.path.exists(outfile))

    def tearDown(self):
        'called multiple times, after every test method'
        if os.path.exists('tmp.png'):
            os.remove('tmp.png')

        if os.path.exists('tmp.csv'):
            os.remove('tmp.csv')



import argparse
import orrey
import sys

#############################
# Create Model Designer     #
#############################
def modelCorrArgs(args):
    parser = argparse.ArgumentParser(
        description='Model Design Correlation Plots')
    parser.add_argument(
        'designfiles', help='Long-form csv model design', nargs='+')
    return parser.parse_args(args)


def modelCorr():
    args = modelCorrArgs(sys.argv[1:])

    mod = orrey.model.ModelWrapper(args.designfiles)
    mod._saveDesign()
    mod._saveFigure()



from setuptools import setup, find_packages


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='orrey',
    version='0.0.1',
    description='Utilities for daily GLM tasks',
    long_description=readme(),
    classifiers=[
        'Development Status :: 1 - Planning',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
    ],
    url='http://github.com/kastman/orrey',
    author='Erik Kastman',
    author_email='erik.kastman@gmail.com',
    license='GPL',
    packages=find_packages(exclude=['docs', 'tests']),
    entry_points={
        'console_scripts': [
            'modelCorr = orrey.cli:modelCorr',
        ]
    },
    scripts=[],
    install_requires=['pandas', 'seaborn', 'moss'],
    test_suite='nose.collector',
    tests_require=['nose'])
